# Package with methods for calculating semantic similarity.
from ._base import SimilarityKernel, SimilarityMatrixCreator
from ._phenomizer import Phenomizer
from ._phenomizer_io import read_ic_mica_data
from ._dummy import DummySimilarityKernel
